@@ -28,9 +28,6 @@ class SharedBackendConf(SharedBackendConfUtils):
     def backendClusterMode(self):
         return self._jconf.backendClusterMode()
 
-    def cloudName(self):
-        return self._get_option(self._jconf.cloudName())
-
     def isH2OReplEnabled(self):
         return self._jconf.isH2OReplEnabled()
 
@@ -55,6 +52,9 @@ class SharedBackendConf(SharedBackendConfUtils):
     def jksAlias(self):
         return self._get_option(self._jconf.jksAlias())
 
+    def sslCACert(self):
+        return self._get_option(self._jconf.sslCACert())
+
     def hashLogin(self):
         return self._jconf.hashLogin()
 
@@ -208,6 +208,9 @@ class SharedBackendConf(SharedBackendConfUtils):
     def icedDir(self):
         return self._get_option(self._jconf.icedDir())
 
+    def cloudName(self):
+        return self._get_option(self._jconf.cloudName())
+
     def restApiTimeout(self):
         return self._jconf.restApiTimeout()
 
@@ -283,6 +286,10 @@ class SharedBackendConf(SharedBackendConfUtils):
         self._jconf.setJksAlias(value)
         return self
 
+    def setSslCACert(self, value):
+        self._jconf.setSslCACert(value)
+        return self
+
     def setHashLoginEnabled(self):
         self._jconf.setHashLoginEnabled()
         return self
