@@ -82,6 +82,7 @@ class H2OGLMClassifier(H2OGLM):
                  maxConfusionMatrixSize=20,
                  computePValues=False,
                  removeCollinearCols=False,
+                 generateScoringHistory=False,
                  modelId=None,
                  nfolds=0,
                  keepCrossValidationModels=True,
@@ -94,6 +95,7 @@ class H2OGLMClassifier(H2OGLM):
                  foldAssignment="AUTO",
                  ignoreConstCols=True,
                  scoreEachIteration=False,
+                 checkpoint=None,
                  stoppingRounds=0,
                  maxRuntimeSecs=0.0,
                  stoppingMetric="AUTO",
@@ -107,6 +109,7 @@ class H2OGLMClassifier(H2OGLM):
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.classification.H2OGLMClassifier", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
+        kwargs = self._updateInitKwargs(kwargs)
         if 'distribution' in kwargs:
             del kwargs['distribution']
             warn("The parameter 'distribution' is deprecated and will be removed in the version 3.34.")
