@@ -2,28 +2,34 @@
 import functools as ft
 
 import h2o
+from h2o.automl._base import H2OAutoMLBaseMixin
+from h2o.automl._h2o_automl_output import H2OAutoMLOutput
 from h2o.base import Keyed
 from h2o.exceptions import H2OResponseError, H2OValueError
 from h2o.frame import H2OFrame
 from h2o.job import H2OJob
-from h2o.model.model_base import ModelBase
 from h2o.utils.shared_utils import check_id
 from h2o.utils.typechecks import assert_is_type, is_type, numeric
 
 
-class H2OAutoML(Keyed):
+class H2OAutoML(H2OAutoMLBaseMixin, Keyed):
     """
     Automatic Machine Learning
 
     The Automatic Machine Learning (AutoML) function automates the supervised machine learning model training process.
-    The current version of AutoML trains and cross-validates 
-    a Random Forest (DRF), 
-    an Extremely-Randomized Forest (DRF/XRT),
-    a random grid of Generalized Linear Models (GLM)
-    a random grid of XGBoost (XGBoost),
-    a random grid of Gradient Boosting Machines (GBM), 
-    a random grid of Deep Neural Nets (DeepLearning), 
-    and 2 Stacked Ensembles, one of all the models, and one of only the best models of each kind.
+    The current version of AutoML trains and cross-validates the following algorithms (in the following order):
+    three pre-specified XGBoost GBM (Gradient Boosting Machine) models,
+    a fixed grid of GLMs,
+    a default Random Forest (DRF),
+    five pre-specified H2O GBMs,
+    a near-default Deep Neural Net,
+    an Extremely Randomized Forest (XRT),
+    a random grid of XGBoost GBMs,
+    a random grid of H2O GBMs,
+    and a random grid of Deep Neural Nets.
+    In some cases, there will not be enough time to complete all the algorithms, so some may be missing from the
+    leaderboard. AutoML then trains two Stacked Ensemble models, one of all the models, and one of only the best
+    models of each kind.
 
     :examples:
     >>> import h2o
@@ -64,15 +70,17 @@ class H2OAutoML(Keyed):
                  project_name=None,
                  exclude_algos=None,
                  include_algos=None,
+                 exploitation_ratio=0,
                  modeling_plan=None,
+                 preprocessing=None,
                  monotone_constraints=None,
-                 algo_parameters=None,
                  keep_cross_validation_predictions=False,
                  keep_cross_validation_models=False,
                  keep_cross_validation_fold_assignment=False,
                  sort_metric="AUTO",
                  export_checkpoints_dir=None,
-                 verbosity="warn"):
+                 verbosity="warn",
+                 **kwargs):
         """
         Create a new H2OAutoML instance.
         
@@ -112,13 +120,12 @@ class H2OAutoML(Keyed):
         :param include_algos: List of character strings naming the algorithms to restrict to during the model-building phase.
           This can't be used in combination with `exclude_algos` param.
           Defaults to ``None``, which means that all appropriate H2O algorithms will be used, if the search stopping criteria allow. Optional.
+        :param exploitation_ratio: The budget ratio (between 0 and 1) dedicated to the exploitation (vs exploration) phase. By default, the exploitation phase is disabled (exploitation_ratio=0) as this is still experimental; to activate it, it is recommended to try a ratio around 0.1. Note that the current exploitation phase only tries to fine-tune the best XGBoost and the best GBM found during exploration.
         :param modeling_plan: List of modeling steps to be used by the AutoML engine (they may not all get executed, depending on other constraints).
           Defaults to None (Expert usage only).
+        :param preprocessing: List of preprocessing steps to run. Only 'target_encoding' is currently supported.
         :param monotone_constraints: Dict representing monotonic constraints.
           Use +1 to enforce an increasing constraint and -1 to specify a decreasing constraint.
-        :param algo_parameters: Dict of ``param_name=param_value`` to be passed to internal models. Defaults to none (Expert usage only).
-          By default, params are set only to algorithms accepting them, and ignored by others.
-          Only following parameters are currently allowed: ``"monotone_constraints"``.
         :param keep_cross_validation_predictions: Whether to keep the predictions of the cross-validation predictions.
           This needs to be set to ``True`` if running the same AutoML object for repeated runs because CV predictions are required to build 
           additional Stacked Ensemble models in AutoML. This option defaults to ``False``.
@@ -134,6 +141,15 @@ class H2OAutoML(Keyed):
         :param verbosity: Verbosity of the backend messages printed during training.
             Available options are None (live log disabled), 'debug', 'info' or 'warn'. Defaults to 'warn'.
         """
+        
+        # early validate kwargs:
+        algo_parameters = None
+        for k in kwargs:
+            if k == 'algo_parameters':
+                algo_parameters = kwargs[k]
+            else:
+                raise TypeError("H2OAutoML got an unexpected keyword argument '%s'" % k)
+        
         # Check if H2O jar contains AutoML
         try:
             h2o.api("GET /3/Metadata/schemas/AutoMLV99")
@@ -164,7 +180,7 @@ class H2OAutoML(Keyed):
 
         assert_is_type(project_name, None, str)
         check_id(project_name, "H2OAutoML")
-        self.project_name = self.build_control["project_name"] = project_name
+        self._project_name = self.build_control["project_name"] = project_name
 
         assert_is_type(nfolds, int)
         assert nfolds >= 0, "nfolds set to " + str(nfolds) + "; nfolds cannot be negative. Use nfolds >=2 if you want cross-valiated metrics and Stacked Ensembles or use nfolds = 0 to disable."
@@ -226,6 +242,9 @@ class H2OAutoML(Keyed):
             assert exclude_algos is None, "Use either include_algos or exclude_algos, not both."
         self.include_algos = self.build_models['include_algos'] = include_algos
 
+        assert_is_type(exploitation_ratio, None, numeric)
+        self.exploitation_ratio = self.build_models['exploitation_ratio'] = exploitation_ratio
+
         assert_is_type(modeling_plan, None, list)
         if modeling_plan is not None:
             supported_aliases = ['all', 'defaults', 'grids']
@@ -269,7 +288,14 @@ class H2OAutoML(Keyed):
         else:
             self.modeling_plan = None
 
-        assert_is_type(algo_parameters, None, dict)
+        assert_is_type(preprocessing, None, [str])  # for now
+        if preprocessing is not None:
+            assert all(p in ['target_encoding'] for p in preprocessing)
+            self.preprocessing = self.build_models['preprocessing'] = [dict(type=p.replace("_", "")) for p in preprocessing]
+        else:
+            self.preprocessing = None
+
+        assert_is_type(monotone_constraints, None, dict)
         if monotone_constraints is not None:
             if algo_parameters is None:
                 algo_parameters = {}
@@ -296,67 +322,35 @@ class H2OAutoML(Keyed):
         assert_is_type(sort_metric, None, str)
         self.sort_metric = self.input_spec['sort_metric'] = sort_metric
 
-
-
     #---------------------------------------------------------------------------
     # Basic properties
     #---------------------------------------------------------------------------
+    @property
+    def project_name(self):
+        return self._project_name
+
+    @project_name.setter
+    def project_name(self, value):
+        self._project_name = value
+
     @property
     def key(self):
         return self._job.dest_key if self._job else self.project_name
 
     @property
     def leader(self):
-        """
-        Retrieve the top model from an H2OAutoML object
-
-        :return: an H2O model
-
-        :examples:
-        >>> # Set up an H2OAutoML object
-        >>> aml = H2OAutoML(max_runtime_secs=30)
-        >>> # Launch an AutoML run
-        >>> aml.train(y=y, training_frame=train)
-        >>> # Get the best model in the AutoML Leaderboard
-        >>> aml.leader
-        """
         return None if self._leader_id is None else h2o.get_model(self._leader_id)
 
     @property
     def leaderboard(self):
-        """
-        Retrieve the leaderboard from an H2OAutoML object
-
-        :return: an H2OFrame with model ids in the first column and evaluation metric in the second column sorted
-                 by the evaluation metric
-
-        :examples:
-        >>> # Set up an H2OAutoML object
-        >>> aml = H2OAutoML(max_runtime_secs=30)
-        >>> # Launch an AutoML run
-        >>> aml.train(y=y, training_frame=train)
-        >>> # Get the AutoML Leaderboard
-        >>> aml.leaderboard
-        """
         return H2OFrame([]) if self._leaderboard is None else self._leaderboard
 
     @property
     def event_log(self):
-        """
-        retrieve the backend event log from an H2OAutoML object
-
-        :return: an H2OFrame with detailed events occurred during the AutoML training.
-        """
         return H2OFrame([]) if self._event_log is None else self._event_log
 
     @property
     def training_info(self):
-        """
-        expose the name/value columns of `event_log` as a simple dictionary, for example `start_epoch`, `stop_epoch`, ...
-        See :func:`event_log` to obtain a description of those key/value pairs.
-
-        :return: a dictionary with event_log['name'] column as keys and event_log['value'] column as values.
-        """
         return dict() if self._training_info is None else self._training_info
 
     @property
@@ -494,22 +488,6 @@ class H2OAutoML(Keyed):
     # Predict with AutoML
     #---------------------------------------------------------------------------
     def predict(self, test_data):
-        """
-        Predict on a dataset.
-
-        :param H2OFrame test_data: Data on which to make predictions.
-
-        :returns: A new H2OFrame of predictions.
-
-        :examples:
-        >>> # Set up an H2OAutoML object
-        >>> aml = H2OAutoML(max_runtime_secs=30)
-        >>> # Launch an H2OAutoML run
-        >>> aml.train(y=y, training_frame=train)
-        >>> # Predict with top model from AutoML Leaderboard on a H2OFrame called 'test'
-        >>> aml.predict(test)
-
-        """
         leader = self.leader
         if leader is None:
             self._fetch()
@@ -518,36 +496,6 @@ class H2OAutoML(Keyed):
             return leader.predict(test_data)
         print("No model built yet...")
 
-    #---------------------------------------------------------------------------
-    # Download POJO/MOJO with AutoML
-    #---------------------------------------------------------------------------
-
-    def download_pojo(self, path="", get_genmodel_jar=False, genmodel_name=""):
-        """
-        Download the POJO for the leader model in AutoML to the directory specified by path.
-
-        If path is an empty string, then dump the output to screen.
-
-        :param path:  An absolute path to the directory where POJO should be saved.
-        :param get_genmodel_jar: if True, then also download h2o-genmodel.jar and store it in folder ``path``.
-        :param genmodel_name: Custom name of genmodel jar
-        :returns: name of the POJO file written.
-        """
-
-        return h2o.download_pojo(self.leader, path, get_jar=get_genmodel_jar, jar_name=genmodel_name)
-
-    def download_mojo(self, path=".", get_genmodel_jar=False, genmodel_name=""):
-        """
-        Download the leader model in AutoML in MOJO format.
-
-        :param path: the path where MOJO file should be saved.
-        :param get_genmodel_jar: if True, then also download h2o-genmodel.jar and store it in folder ``path``.
-        :param genmodel_name: Custom name of genmodel jar
-        :returns: name of the MOJO file written.
-        """
-
-        return ModelBase.download_mojo(self.leader, path, get_genmodel_jar, genmodel_name)
-
     #-------------------------------------------------------------------------------------------------------------------
     # Overrides
     #-------------------------------------------------------------------------------------------------------------------
@@ -665,7 +613,8 @@ def get_automl(project_name):
     :param str project_name:  A string indicating the project_name of the automl instance to retrieve.
     :returns: A dictionary containing the project_name, leader model, leaderboard, event_log.
     """
-    return H2OAutoML._fetch_state(project_name)
+    state = H2OAutoML._fetch_state(project_name)
+    return H2OAutoMLOutput(state)
 
 
 def get_leaderboard(aml, extra_columns=None):
@@ -678,6 +627,7 @@ def get_leaderboard(aml, extra_columns=None):
         - 'ALL': adds all columns below.
         - 'training_time_ms': column providing the training time of each model in milliseconds (doesn't include the training of cross validation models).
         - 'predict_time_per_row_ms`: column providing the average prediction time by the model for a single row.
+        - 'algo': column providing the algorithm name for each model.
     :return: An H2OFrame representing the leaderboard.
     :examples:
     >>> aml = H2OAutoML(max_runtime_secs=30)
@@ -686,5 +636,5 @@ def get_leaderboard(aml, extra_columns=None):
     >>> lb_custom = h2o.automl.get_leaderboard(aml, ['predict_time_per_row_ms', 'training_time_ms'])
     >>> lb_custom_sorted = lb_custom.sort(by='predict_time_per_row_ms')
     """
-    assert_is_type(aml, H2OAutoML)
+    assert_is_type(aml, H2OAutoML, H2OAutoMLOutput)
     return H2OAutoML._fetch_leaderboard(aml.key, extra_columns)
